<?php

/**
 * @file
 * Misc functions that hardly change.
 */

use Drupal\Component\Utility\String;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Unicode;
use Drupal\Core\Url;

/**
 * A helper function to generate a list of blocks from a specified region.
 *
 * @param array $region
 *   The string identifier for a Ultimenu region. e.g. "ultimenu_main_about"
 *
 * @return array
 *   The renderable array of blocks within the region.
 */
function ultimenu_get_blocks_by_region($region) {
  $build = array();
  $theme = \Drupal::config('system.theme')->get('default');
  $blocks = entity_load_multiple_by_properties('block', array('theme' => $theme, 'region' => $region));

  if ($blocks) {
    uasort($blocks, 'Drupal\block\Entity\Block::sort');
    foreach ($blocks as $key => $block) {
      if ($block->access('view')) {
        $build[$key] = entity_view($block, 'block');
      }
    }
  }

  return $build;
}

/**
 * Returns the array of enabled Ultimenu regions based on enabled settings.
 */
function ultimenu_regions_enabled($delta = NULL) {
  $regions = &drupal_static(__FUNCTION__);

  if (!isset($regions)) {
    $regions     = array();
    $regions_all = ultimenu_regions($delta);

    // First limit to enabled regions from the settings.
    if (($regions_enabled = ultimenu_get_settings('regions')) !== NULL) {
      foreach (array_filter($regions_enabled) as $enabled) {
        // We must depends on enabled menu items as always.
        // A disabled menu item will automatically drop its Ultimenu region.
        if (array_key_exists($enabled, $regions_all)) {
          $regions[$enabled] = $regions_all[$enabled];
        }
      }
    }
  }
  return $regions;
}

/**
 * The array of available Ultimenu regions based on enabled menu items.
 *
 * @return array
 *   An array of regions definition dependent on available main-menu items.
 *
 * @see ultimenu_load_menu_tree()
 */
function ultimenu_regions($delta = NULL) {
  $regions = &drupal_static(__FUNCTION__);
  if (!isset($regions)) {
    $blocks      = ultimenu_get_settings('blocks');
    $goodies     = ultimenu_get_settings('goodies');
    $is_mlid     = !empty($goodies['ultimenu-mlid']) ? TRUE : FALSE;
    $menu_blocks = is_array($blocks) ? array_filter($blocks) : array($blocks);
    $menus       = array();

    if ($delta) {
      $menus[$delta] = ultimenu_load_menu_tree($delta);
    }
    else {
      foreach ($menu_blocks as $menu_name => $title) {
        $menus[$menu_name] = ultimenu_load_menu_tree($menu_name);
      }
    }

    // Allow alteration.
    \Drupal::moduleHandler()->alter('ultimenu_menus_info', $menus);

    $regions = array();
    foreach ($menus as $menu_name => $tree) {
      $tree = \Drupal::menuTree()->build($tree);
      foreach ($tree['#items'] as $key => $item) {
        $name_id = ultimenu_truncate($menu_name);
        $name_id_nice = str_replace("_", " ", $name_id);

        if ($item['title']) {
          $menu_title = String::checkPlain($item['title']);
          $region_key = ultimenu_get_region_key($menu_name, $key, $menu_title);
          $regions[$region_key] = "Ultimenu:$name_id_nice: $menu_title";
          $regions = array_unique($regions);
        }
      }
    }

    // Allow alteration.
    \Drupal::moduleHandler()->alter('ultimenu_regions_info', $regions);
  }

  return $regions;
}

/**
 * Gets Ultimenu regions from the default theme .info.yml.
 */
function ultimenu_parse_theme_info() {
  $regions = &drupal_static(__FUNCTION__);

  if (!isset($regions)) {
    $theme = \Drupal::config('system.theme')->get('default');
    $file  = drupal_get_path('theme', $theme) . '/' . $theme . '.info.yml';

    // Parse theme .info.yml file.
    $info = \Drupal::service('info_parser')->parse($file);

    $regions = array();
    foreach ($info['regions'] as $key => $region) {
      if (array_key_exists($key, ultimenu_regions())) {
        $regions[$key] = $region;
      }
    }
  }
  return $regions;
}

/**
 * Colllects unwanted Ultimenu regions for removal stored in the theme .info.yml.
 *
 * When a menu item disabled or deleted, relevant dynamic Ultimenu regions
 * removed from the system, but theme .info.yml has a copy stored there.
 * System will always keep and display the unwanted.
 * Either manual deletion from .info.yml, or a force removal if so configured.
 *
 * @return array|bool
 *   The array of unwanted Ultimenu regions from theme .info.yml, or FALSE.
 */
function ultimenu_remove_region_theme_info() {
  $goodies = ultimenu_get_settings('goodies');
  if (!empty($goodies['force-remove-region']) && $theme_regions = ultimenu_parse_theme_info()) {
    return $theme_regions;
  }
  return FALSE;
}

/**
 * Simplify menu names or menu item titles for region key.
 *
 * If region key is to use menu item title:
 * Region key: ultimenu_LOOOOOOOOOOOOOONGMENUNAME_LOOOOOOOOOOOOOOOOOONGMENUITEM.
 * If region key is to use unfriendly key UUID, we'll only care for menu name.
 * Region key: ultimenu_LOOOOOOOOOOOOOONGMENUNAME_1c2d3e4
 *
 * @param string $string
 *   The Menu name or menu item title
 *
 * @param int $max_length
 *   The amount of characters to truncate.
 *
 * @return string
 *   The truncated menu properties ready to use for region key.
 */
function ultimenu_truncate($string, $max_length = 28) {
  // Transliterate the string.
  $langcode = \Drupal::languageManager()->getLanguage()->langcode;
  $string = \Drupal::service('transliteration')->transliterate($string, $langcode);

  // Decodce it.
  $transformed = String::decodeEntities($string);
  $transformed = Unicode::strtolower(str_replace(array('menu-', '-menu'), '', $transformed));
  $transformed = preg_replace('/[\W\s]+/', '_', $transformed);

  // Trim trailing underscores.
  $transformed = trim($transformed, '_');
  $transformed = Unicode::truncate($transformed, $max_length, TRUE, FALSE);
  return $transformed;
}

/**
 * Get all available menus, excluding some admin menus.
 */
function ultimenu_get_menus() {
  $menus = &drupal_static(__FUNCTION__);
  if (!isset($menus)) {
    $available_menus = menu_ui_get_menus();
    // $available_menus = entity_load_multiple('menu');
    $exclude_menus = array(
      'devel' => t('Development'),
      'admin' => t('Administration'),
      'tools' => t('Tools'),
    );

    $menus = array_diff_key($available_menus, $exclude_menus);
  }
  return $menus;
}

/**
 * Collects enabled Ultimenu block items.
 */
function ultimenu_get_blocks() {
  $items = &drupal_static(__FUNCTION__);
  if (!isset($items)) {
    $items = array();
    $menus = ultimenu_get_menus();
    foreach ($menus as $delta => $name) {
      if (ultimenu_blocks_enabled($delta)) {
        $items[$delta] = t("@name", array('@name' => $name));
      }
    }
  }
  return $items;
}

/**
 * Check for an Ultimenu block enabled by settings.
 */
function ultimenu_blocks_enabled($delta) {
  $blocks = ultimenu_get_settings('blocks');
  if (!empty($blocks[$delta])) {
    return TRUE;
  }
  return FALSE;
}

/**
 * Retrieves stored CSS files for Ultimenu skin options.
 *
 * @param bool $refresh
 *   If TRUE, reload the CSS and flush the cached version.
 *
 * @return array
 *   An array of available CSS files.
 */
function ultimenu_get_skins($refresh = FALSE) {
  $skins = &drupal_static(__FUNCTION__);

  if (!isset($skins) || $refresh) {
    if (!$refresh && $cache = \Drupal::cache()->get('ultimenu:skin')) {
      $skins = $cache->data;
    }
    else {
      $theme_default = \Drupal::config('system.theme')->get('default');
      $theme_skin    = drupal_get_path('theme', $theme_default) . '/css/ultimenu';
      $custom_skin   = ultimenu_get_settings('skins');
      $module_skin   = drupal_get_path('module', 'ultimenu') . '/skins';
      $mask          = '/.css$/';

      $files = array();
      if (is_dir($module_skin)) {
        foreach (file_scan_directory($module_skin, $mask) as $filename => $file) {
          $files[$filename] = $file;
        }
      }
      if (!empty($custom_skin) && is_dir($custom_skin)) {
        foreach (file_scan_directory($custom_skin, $mask) as $filename => $file) {
          $files[$filename] = $file;
        }
      }
      if (is_dir($theme_skin)) {
        foreach (file_scan_directory($theme_skin, $mask) as $filename => $file) {
          $files[$filename] = $file;
        }
      }
      if ($files) {
        $skins = array();
        foreach ($files as $file) {
          $uri  = $file->uri;
          $name = $file->name;

          // Simplify lengthy deep directory structure.
          if (strpos($uri, $module_skin) !== FALSE) {
            $uri = "module|" . $name;
          }
          elseif (strpos($uri, $custom_skin) !== FALSE) {
            $uri = "custom|" . $name;
          }
          elseif (strpos($uri, $theme_skin) !== FALSE) {
            $uri = "theme|" . $name;
          }

          // Convert file name to CSS friendly for option label and styling.
          $skins[$uri] = Html::cleanCssIdentifier(Unicode::strtolower($name));
        }
        \Drupal::cache()->set('ultimenu:skin', $skins);
      }
    }
  }
  return $skins;
}

/**
 * A reversed process to convert an option into a full CSS skin path.
 *
 * This silly reversion ensures the setting will be intact when moving around
 * CSS files, or theme and module directory.
 *
 * @param string $path
 *   The path that should be converted to full CSS path.
 *
 * @return string
 *   The CSS path containing ultimenu skins.
 */
function ultimenu_get_skin_path($path) {
  $css_path = &drupal_static(__FUNCTION__);

  if (!$css_path) {
    $theme_default = \Drupal::config('system.theme')->get('default');
    $theme_skin    = drupal_get_path('theme', $theme_default) . '/css/ultimenu';
    $custom_skin   = ultimenu_get_settings('skins');
    $module_skin   = drupal_get_path('module', 'ultimenu') . '/skins';
    $css_path      = '';

    $parts = explode("|", $path);
    if (strpos($path, "module|") !== FALSE) {
      $css_path = $module_skin . '/' . $parts[1] . '.css';
    }
    elseif (strpos($path, "custom|") !== FALSE) {
      $css_path = $custom_skin . '/' . $parts[1] . '.css';
    }
    elseif (strpos($path, "theme|") !== FALSE) {
      $css_path = $theme_skin . '/' . $parts[1] . '.css';
    }

    if ($css_path) {
      $css_path = Url::fromUri('base://' . $css_path);
      $css_path = \Drupal::urlGenerator()->generateFromPath($css_path);
    }
  }

  return $css_path;
}

/**
 * Gets the skin basename.
 *
 * @param string $skin
 *   The skin in the format: provider|ultimenu--skin-base-name.
 *
 * @return string
 *   The skin basename.
 */
function ultimenu_get_skin_basename($skin) {
  list($provider, $skin_name) = array_pad(array_map('trim', explode("|", $skin, 2)), 2, NULL);
  $skin_basename = str_replace("ultimenu--", "", $skin_name);
  $skin_basename = str_replace("-", "_", $skin_basename);
  return $skin_basename;
}

/**
 * Gets the region key.
 *
 * @param string $key
 *   The menu item key can be UUID or menu item title.
 *
 * @return string
 *   The PHP region key name based on UUID, or menu item title.
 */
function ultimenu_get_region_key($menu_name, $key, $title, $max_length = 28, $is_uuid = FALSE) {
  $goodies = ultimenu_get_settings('goodies');
  $menu_name = ultimenu_truncate($menu_name, $max_length);
  list($prefix, $uuid) = array_pad(array_map('trim', explode(":", $key, 2)), 2, NULL);
  list($mlid, $suffix) = array_pad(array_map('trim', explode("-", $uuid, 2)), 2, NULL);
  $menu_item = $goodies['ultimenu-mlid'] ? $mlid : ultimenu_truncate(strip_tags($title), $max_length);
  $region_key = $is_uuid ? $mlid : 'ultimenu_' . $menu_name . '_' . $menu_item;

  return $region_key;
}

/**
 * A wrapper function for configFactory().
 *
 * @param string $setting_name
 *   The name of the setting.
 *
 * @param string $default
 *   The default value if specified.
 *
 * @return mixed
 *   The value of the setting, or default.
 */
function ultimenu_get_settings($setting_name = NULL, $default = NULL) {
  $cache = &drupal_static(__FUNCTION__);

  if (empty($cache)) {
    $config = \Drupal::configFactory()->getEditable('ultimenu.settings');
    if ($config->get($setting_name) && ($setting = $config->get($setting_name)) !== NULL) {
      return $setting;
    }
  }
  else {
    if (isset($cache[$setting_name])) {
      return $cache[$setting_name];
    }
  }
  return $default;
}
