<?php

/**
 * @file
 * Contains \Drupal\ultimenu\Plugin\Block\UltimenuBlock.
 */

namespace Drupal\ultimenu\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Url;

/**
 * Provides an 'Ultimenu' block.
 *
 * @Block(
 *  id = "ultimenu_block",
 *  admin_label = @Translation("Ultimenu block"),
 *  category = @Translation("Ultimenu"),
 *  deriver = "Drupal\ultimenu\Plugin\Derivative\UltimenuBlock"
 * )
 */
class UltimenuBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return array(
      'skin' => 'module|ultimenu--basic',
      'orientation' => 'ultimenu--htb',
    );
  }

  /**
   * Overrides \Drupal\block\BlockBase::blockForm().
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $account = \Drupal::currentUser();

    if ($account->hasPermission('administer ultimenu')) {

      $form['skin'] = array(
        '#type' => 'select',
        '#title' => $this->t('Ultimenu skin'),
        '#default_value' => $this->configuration['skin'],
        '#options' => ultimenu_get_skins(),
        '#empty_option' => $this-> t('- None -'),
        '#description' => $this->t('Choose the skin for this block. You can supply custom skins at <a href="@ultimenu_settings" target="_blank">Ultimenu settings</a>. The skin can be made specific to this block using the proper class by each menu name. Be sure to <a href="@clear" target="_blank">clear the cache</a> if trouble to see the new skin applied.', array('@ultimenu_settings' => \Drupal::url('ultimenu.settings'), '@clear' => \Drupal::url('system.performance_settings'))),
      );

      $form['orientation'] = array(
        '#type' => 'select',
        '#title' => $this->t('Flyout orientation'),
        '#default_value' => $this->configuration['orientation'],
        '#options' => array(
          'ultimenu--htb' => t('Horizontal to bottom'),
          'ultimenu--htt' => t('Horizontal to top'),
          'ultimenu--vtl' => t('Vertical to left'),
          'ultimenu--vtr' => t('Vertical to right'),
        ),
        '#description' => $this->t('Choose the orientation of the flyout, depending on the placement. At sidebar left, <strong>Vertical to right</strong>. At header, <strong>Horizontal to bottom</strong>. At footer, <strong>Horizontal to top</strong>'),
      );
    }
    return $form;
  }

  /**
   * Overrides \Drupal\block\BlockBase::blockSubmit().
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['skin'] = $form_state->getValue('skin');
    $this->configuration['orientation'] = $form_state->getValue('orientation');

    // Invalidate the library discovery cache to update new skin derivatives.
    \Drupal::service('library.discovery')->clearCachedDefinitions();

    // Invalidate the block cache to update ultimenu block-based derivatives.
    \Drupal::service('plugin.manager.block')->clearCachedDefinitions();
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    // Reconsider whether to use ':' as separator for derivative plugins
    // @see https://www.drupal.org/node/2035345
    // Derivatives are prefixed with 'ultimenu-'.
    $id = $this->getDerivativeId();
    $menu_name = substr($id, 9);

    $provider      = '';
    $skin_name     = '';
    $skin_css_path = '';
    $skin_basename = '';
    $skin          = $this->configuration['skin'];
    $orientation   = $this->configuration['orientation'];

    // Load the specified block skin.
    if (!empty($skin)) {
      $skin_css_path = ultimenu_get_skin_path($skin);
      // Fetch the skin file name from the setting.
      list($provider, $skin_name) = array_pad(array_map('trim', explode("|", $skin, 2)), 2, NULL);
      $skin_basename = ultimenu_get_skin_basename($skin);
    }

    // Provide the settings for ultimenu_tree_output().
    $configs = array(
      'skin'          => $skin,
      'orientation'   => $orientation,
      'delta'         => $menu_name,
      'menu_name'     => $menu_name,
      'skin_name'     => $skin_name,
      'skin_provider' => $provider,
    );

    // Build the ultimenu tree.
    $tree = ultimenu_build_menu_tree($menu_name, $configs);

    // Allow alteration of the tree before we begin operations on it.
    // @see https://drupal.org/node/1894902
    \Drupal::moduleHandler()->alter('ultimenu_tree', $tree);

    // Build the block.
    $build = array();
    if ($tree) {
      $build['content'] = array(
        '#theme'  => 'ultimenu',
        '#items'  => $tree,
        '#config' => $configs,
        '#delta'  => $menu_name,
      );
      $build['#attached']['library'][] = 'ultimenu/ultimenu';
      if ($skin_basename) {
        $build['#attached']['library'][] = 'ultimenu/ultimenu.skin.' . $skin_basename;
      }
    }

    return $build;
  }
}
